-- 游戏 agent 服务
local skynet = require "skynet"

local code = require "code"

local jmService = require "jmService"
local nodeUtil = require("utils.nodeUtil")
local lib_queue = require("lib_queue")
local util_event = require("utils.util_event")

local mapItf2Module = {}

local root = {
	mapUid2Agent = {}
}

-- 登录
function root.login(uid, session)
	root.mapUid2Agent[uid] = session
	-- 通知 agentmgr
	skynet.call(".srvAgentMgr", "lua", "login", uid, session)

	nodeUtil:user_master_band_node(uid)

	return true
end
-- 登出
function root.logout(uid, session)
	root.mapUid2Agent[uid] = nil

	skynet.call(".srvAgentMgr", "lua", "logout", uid, session)
	nodeUtil:user_master_unband_node(uid)

	return true
end

-- 回应请求协议
local function l_c2s_response(uid, protoName, errCode, res, ...)
	res = res or {}
	res.code = errCode

	local session = root.mapUid2Agent[uid]
	if session == nil or session.gateNode == nil or session.gateAgent == nil then
		return
	end

	nodeUtil:send_to_node(session.gateNode, session.gateAgent, "c2s_response", uid, protoName, res, ...)
end

-- 格式化协议
local function l_get_interface_and_func(protoName)
	-- return protoName:match "([^_]*)_(.*)"

	local data = string.split(protoName, "_")
	local itfName = table.remove(data, 1)

	local funcName = table.concat(data, "_")
	return itfName, funcName
end

-- 获取接口对象
local function l_get_interface_module(itfName)
	local fileName = string.format("interface.%s", itfName)
	mapItf2Module[itfName] = mapItf2Module[itfName] or require(fileName)
	return mapItf2Module[itfName]
end

-- 处理客户端协议数据
function root.c2s_forward(uid, protoName, args, ...)
	local session = root.mapUid2Agent[uid]
	if session == nil then
		log.warning("c2s_forward not find role! uid:%s", uid)
		return l_c2s_response(uid, protoName, code.INNER_SERVER_ERROR)
	end

	local itfName, funcName = l_get_interface_and_func(protoName)
	local itfModule = l_get_interface_module(itfName)

	if itfModule == nil then
		log.error("dont find module ! itfName:%s, funcName:%s", itfName, funcName)
		return l_c2s_response(uid, protoName, code.INNER_SERVER_ERROR, nil, ...)
	end
	local func = itfModule[funcName]
	if not func then
		log.error("dont find func ! itfName:%s, funcName:%s", itfName, funcName)
		return l_c2s_response(uid, protoName, code.INNER_SERVER_ERROR, nil, ...)
	end

	local ok, errCode, rs =
		xpcall(
		func,
		function()
			skynet.error(debug.traceback())
		end,
		itfModule,
		uid,
		args
	)
	if not ok then
		log.error(string.format("itfName:%s, funcName:%s, args:%s", itfName, funcName, tostring(args)))
		return l_c2s_response(uid, protoName, code.INNER_SERVER_ERROR, nil, ...)
	end

	l_c2s_response(uid, protoName, errCode, rs, ...)

	-- 协议处理完后进行处理
	local afterFuncName = string.format("after_%s", funcName)
	local afterFunc = itfModule[afterFuncName]
	if not afterFunc then
		return
	end

	ok, errCode, rs =
		xpcall(
		afterFunc,
		function()
			skynet.error(debug.traceback())
		end,
		itfModule,
		uid,
		args
	)
	if not ok then
		log.error(string.format("after itfName:%s, funcName:%s, args:%s", itfName, funcName, tostring(args)))
		return
	end
end

function root.doCmd(uid, itfName, cmd, args)
	local itfModule = l_get_interface_module(itfName)
	if itfModule == nil then
		log.error("doCmd dont find module ! itfName:%s, cmd:%s", itfName, cmd)
		return 400
	end

	local func = itfModule[cmd]
	if not func then
		log.error("doCmd dont find func ! itfName:%s, cmd:%s", itfName, cmd)
		return 401
	end

	local ok, errCode, rs =
		xpcall(
		func,
		function()
			skynet.error(debug.traceback())
		end,
		uid,
		args
	)

	if not ok then
		log.error(string.format("doCmd itfName:%s, cmd:%s, args:%s", itfName, cmd, tostring(args)))
		return
	end

	return errCode, rs
end

-- 玩家事件
function root.user_trigge_event(uid, eventId, evtParams)
	log.debug("user_trigge_event uid[%s] eventId[%s] eventParams[%s]", uid, eventId, tostring(evtParams))
	lib_queue.critical_section(uid)(util_event.user_event, util_event, uid, eventId, evtParams)
end

-- 启动服务
function root.onStart()
	return true
end

-- 服务关闭
function root.onStop()
	-- 向agent管理服务发送退出成功消息
	skynet.send(".srvAgentMgr", "lua", "exit", skynet.self())
	return true
end

function root.recycle()
	skynet.exit()
end

-- 更新配置表
function root.update_config(...)
	jmService.update_config(...)
	util_event:init()
end

-- 更新逻辑
function root.update_logic(...)
	jmService.update_logic(...)
	util_event:init()
end

-- 在线玩家数
function root.get_player_count()
	return #root.mapUid2Agent
end
-- 打印 - 玩家列表
function root.print_uid_list()
	local uidList = {}
	for uid, _ in pairs(root.mapUid2Agent) do
		table.insert(uidList, uid)
	end
	log.warning("print_uid_list uidList:%s", tostring(uidList))
end

jmService.start(root, "srvAgent_" .. skynet.self())
